#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "osint.h"

#define LOADER_BAUD 2000000
#define DEFAULT_BAUD 115200

#if defined(__CYGWIN__) || defined(__MINGW32__) || defined(__MINGW64__)
  #define PORT_PREFIX "com"
#elif defined(__APPLE__)
  #define PORT_PREFIX "/dev/cu.usbserial"
#else
  #define PORT_PREFIX "/dev/ttyUSB"
#endif

char *MainLoader =
"00 fe 65 fd 00 00 8c fc 20 7e 65 fd 24 08 60 fd 24 28 60 fd 28 1e 60 fd 08 06 dc fc 4f 7e 74 fd 01 24 84 f0 28 20 60 fd 18 24 44 f0 13 24 60 fd f6 23 6c fb 00 00 7c fc 00 00 ec fc";

static char buffer[1024];

/* Usage - display a usage message and exit */
static void Usage(void)
{
printf("\
loadp2 - a loader for the propeller 2 - version 0.003, 2017-04-23\n\
usage: loadp2\n\
         [ -p port ]               serial port\n\
         [ -b baud ]               baud rate (default is %d)\n\
         [ -t ]                    enter terminal mode after running the program\n\
         [ -T ]                    enter PST-compatible terminal mode\n\
         [ -? ]                    display a usage message and exit\n\
         file                      file to load\n", DEFAULT_BAUD);
    exit(1);
}

void txval(int val)
{
    sprintf(buffer, " %2.2x %2.2x %2.2x %2.2x",
        val&255, (val >> 8) & 255, (val >> 16) & 255, (val >> 24) & 255);
    tx((uint8_t *)buffer, strlen(buffer));
}

int loadfile(char *fname)
{
    FILE *infile;
    int num, size;

    infile = fopen(fname, "rb");
    if (!infile)
    {
        printf("Could not open %s\n", fname);
        return 1;
    }
    fseek(infile, 0, SEEK_END);
    size = ftell(infile);
    fseek(infile, 0, SEEK_SET);
    printf("Loading %s - %d bytes\n", fname, size);
    hwreset();
    msleep(50);
    tx((uint8_t *)"> Prop_Hex 0 0 0 0 ", 19);
    tx((uint8_t *)MainLoader, strlen(MainLoader));
    txval(3*80000000/LOADER_BAUD/2-6);
    txval(80000000/LOADER_BAUD-6);
    txval(size);
    tx((uint8_t *)"~", 1);
    msleep(100);
    while ((num=fread(buffer, 1, 1024, infile)))
    {
        tx((uint8_t *)buffer, num);
    }
    msleep(50);
    printf("%s loaded\n", fname);
    return 0;
}

int findp2(char *portprefix, int baudrate)
{
    int i, num;
    char Port[20];
    char buffer[101];

    printf("Searching serial ports for a P2\n");
    for (i = 0; i < 20; i++)
    {
        sprintf(Port, "%s%d", portprefix, i);
        if (serial_init(Port, baudrate))
        {
            hwreset();
            msleep(50);
            tx((uint8_t *)"> Prop_Chk 0 0 0 0  ", 20);
            msleep(50);
            num = rx_timeout((uint8_t *)buffer, 100, 10);
            if (num >= 0) buffer[num] = 0;
            else buffer[0] = 0;
            if (!strncmp(buffer, "\r\nProp_Ver ", 11))
            {
                printf("P2 version %c found on serial port %s\n", buffer[11], Port);
                return 1;
            }
            serial_done();
        }
    }
    return 0;
}

int main(int argc, char **argv)
{
    int i;
    int runterm = 0;
    int pstmode = 0;
    char *fname = 0;
    char *port = 0;
    int baudrate = DEFAULT_BAUD;

    // Parse the command-line parameters
    for (i = 1; i < argc; i++)
    {
        if (argv[i][0] == '-')
        {
            if (argv[i][1] == 'p')
            {
                if(argv[i][2])
                    port = &argv[i][2];
                else if (++i < argc)
                    port = argv[i];
                else
                    Usage();
            }
            else if (argv[i][1] == 'b')
            {
                if(argv[i][2])
                    baudrate = atoi(&argv[i][2]);
                else if (++i < argc)
                    baudrate = atoi(argv[i]);
                else
                    Usage();
            }
            else if (argv[i][1] == 't')
                runterm = 1;
            else if (argv[i][1] == 'T')
                runterm = pstmode = 1;
        }
        else
        {
            if (fname) Usage();
            fname = argv[i];
        }
    }

    if (!fname) Usage();
    if (!port)
    {
        if (!findp2(PORT_PREFIX, LOADER_BAUD))
        {
            printf("Could not find a P2\n");
            exit(1);
        }
    }
    else if (1 != serial_init(port, LOADER_BAUD))
    {
        printf("Could not open port %s\n", argv[1]);
        exit(1);
    }

    if (loadfile(fname))
    {
        serial_done();
        exit(1);
    }

    if (runterm)
    {
        serial_baud(baudrate);
        printf("[ Entering terminal mode.  Press ESC to exit. ]\n");
        terminal_mode(1,pstmode);
    }

    serial_done();
    return 0;
}
